package jp.co.asterisk.asreader.a23d.demoapp;

import android.app.AlertDialog;
import android.content.pm.PackageManager;
import android.os.Bundle;

import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.navigation.fragment.NavHostFragment;

import android.util.Log;
import android.view.View;
import android.widget.Toast;

import java.util.ArrayList;
import java.util.Map;
import java.util.Objects;

import jp.co.asterisk.asreader.a23d.demoapp.databinding.ActivityMainBinding;
import jp.co.asterisk.asreader.a23d.demoapp.fragments.BaseFragment;
import jp.co.asterisk.asreader.a23d.demoapp.fragments.MenuFragment;
import jp.co.asterisk.asreader.a23d.demoapp.fragments.InfoFragment;
import jp.co.asterisk.asreader.a23d.demoapp.fragments.BarcodeScanFragment;
import jp.co.asterisk.asreader.a23d.demoapp.fragments.ProgressDialogFragment;
import jp.co.asterisk.asreader.a23d.demoapp.fragments.SettingFragment;
import jp.co.asterisk.asreader.a23d.demoapp.util.SharedPreferencesHelper;
import jp.co.asterisk.asreader.a23d.demoapp.util.TagData;
import jp.co.asterisk.asreader.a23d.sdk.AsReader;
import jp.co.asterisk.asreader.a23d.sdk.barcodemanager.BarcodeConst;
import jp.co.asterisk.asreader.a23d.sdk.devicemanager.*;

public class MainActivity extends AppCompatActivity implements DeviceManagerCallback {

    final String TAG = "MainActivity";
    private ActivityMainBinding binding;

    private ArrayList<TagData> tagDataList;
    private static Toast toast = null;

    @Override
    protected void onStop() {
        super.onStop();
        Log.d(TAG, "onStop");
    }

    @Override
    protected void onResume() {
        super.onResume();
        Log.d(TAG, "onResume");
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        Log.d(TAG, "onDestroy");
    }


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        binding = ActivityMainBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());
//        binding.toolbar.setTitle("");
        setSupportActionBar(binding.toolbar);

        binding.backButton.setOnClickListener((View view) -> {
            Fragment currentFragment = getCurrentFragment();
            if (!(currentFragment instanceof MenuFragment)) {
                assert currentFragment != null;
                NavHostFragment.findNavController(currentFragment).popBackStack();
            }
        });

        binding.rightButton.setOnClickListener((View view) -> {
            Fragment currentFragment = getCurrentFragment();
            if (currentFragment instanceof BaseFragment) {
                ((BaseFragment)currentFragment).onRightButtonClick();
            }
        });

        SharedPreferencesHelper.getInstance().initialize(getApplicationContext());

        tagDataList = new ArrayList<>();

        AsReader.getInstance().getDeviceManager().setCallback(this);
        AsReader.getInstance().initialize(getApplicationContext());

        AsReader.getInstance().getBarcodeManager().setEncodeMode(BarcodeConst.QREncodeMode.AUTO);

        AsReader.getInstance().setLog(true);
    }

    public void setTitle(String title) {
        binding.titleTextView.setText(title);
    }

    public void enableActionBar(boolean enable){
        if(enable) {
            Objects.requireNonNull(getSupportActionBar()).show();
        } else {
            Objects.requireNonNull(getSupportActionBar()).hide();
        }
    }

    public void setBackButtonVisibility(int visibility) {
        binding.backButton.setVisibility(visibility);
    }

    public void setRightButtonVisibility(int visibility) {
        binding.rightButton.setVisibility(visibility);
    }

    public void setRightButtonEnabled(boolean enabled) {
        binding.rightButton.setEnabled(enabled);
    }

    public void setRightButtonText(String text) {
        binding.rightButton.setText(text);
    }

    public ArrayList<TagData> getTagDataList() {
        return tagDataList;
    }

    private Fragment getCurrentFragment() {
        Fragment navHostFragment = getSupportFragmentManager().findFragmentById(R.id.nav_host_fragment);
        return (navHostFragment != null) ? navHostFragment.getChildFragmentManager().getFragments().get(0) : null;
    }

    public String getAppVersion() {
        String versionName;
        String packageName = getPackageName();

        // Get Application Version
        try {
            versionName = getPackageManager().getPackageInfo(packageName, PackageManager.GET_META_DATA).versionName;
        } catch (PackageManager.NameNotFoundException e) {
            versionName = "";
        }

        return versionName;
    }

    private void showToast(String message) {
        if(toast != null) toast.cancel();
        toast = Toast.makeText(this, message, Toast.LENGTH_SHORT);
        toast.show();
    }

    public void showExitAlert() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this, android.R.style.Theme_DeviceDefault_Light_Dialog);
        builder.setTitle(" Alert  ");
        builder.setMessage("Do you want to exit the program?");
        builder.setPositiveButton("Exit", (arg0, arg1) -> {
            AsReader.getInstance().getDeviceManager().setPowerWithOption(false, false, false,false, false, false);
            android.os.Process.killProcess(android.os.Process.myPid());
        });

        builder.setNegativeButton("Cancel", (dialog, which) -> {

        });

        builder.show();
    }

    public void removeDialog() {
        FragmentManager mgr = getSupportFragmentManager();
        Fragment fragment = mgr.findFragmentByTag("ProgressDialogFragment");
        if( fragment instanceof ProgressDialogFragment ) {
            ((ProgressDialogFragment)fragment).dismiss();
        }

    }
    @Override
    public void onBackPressed() {
        Fragment currentFragment = getCurrentFragment();
        if (currentFragment instanceof MenuFragment) {
            ((BaseFragment) currentFragment).onBackPressed();
//            showExitAlert();
        } else {
            assert currentFragment != null;
            NavHostFragment.findNavController(currentFragment).popBackStack();
        }
    }

    // *******************************************************
    //
    //  DeviceManagerCallback
    //
    // *******************************************************
    @Override
    public void onConnect(boolean isConnect) {
        Log.d(TAG, "onConnect: " + isConnect);
        runOnUiThread(() -> {
            Fragment currentFragment = getCurrentFragment();
            if (currentFragment instanceof BaseFragment) {
                ((BaseFragment) currentFragment).onConnect(isConnect);
            }
        });
    }

    @Override
    public void onCompletedPreparation(boolean isPowerOn) {
        Log.d(TAG, "onCompletedPreparation: " + isPowerOn);
        runOnUiThread(() -> {
            String msg = "onCompletedPreparation:" + isPowerOn;
            showToast(msg);
            Fragment currentFragment = getCurrentFragment();

            if (currentFragment instanceof BarcodeScanFragment) {
                ((BarcodeScanFragment) currentFragment).onCompletedPreparation(isPowerOn);
            } else if (currentFragment instanceof SettingFragment) {
                ((SettingFragment) currentFragment).onCompletedPreparation(isPowerOn);
            }
        });
    }

    @Override
    public void onSetPowerOnSuccess() {

    }

    @Override
    public void onBatteryStateReceived(int battery) {
        runOnUiThread(() -> {
            Fragment currentFragment = getCurrentFragment();
            if (currentFragment instanceof BarcodeScanFragment) {
                ((BarcodeScanFragment) currentFragment).onBatteryStateReceived(battery);
            }
        });
    }

    @Override
    public void onSetReaderSettingSuccess() {
        Log.d(TAG, "onSetReaderSettingSuccess");
        runOnUiThread(() -> {
            Fragment currentFragment = getCurrentFragment();
            if (currentFragment instanceof SettingFragment) {
                ((SettingFragment) currentFragment).onSetReaderSettingSuccess();
            }
        });
    }

    @Override
    public void onBarcodePowerOnBeepChanged(boolean isPowerOnBeep) {
        Log.d(TAG, "onBarcodePowerOnBeepChanged: " + isPowerOnBeep);
        runOnUiThread(() -> {
            Fragment currentFragment = getCurrentFragment();
            if (currentFragment instanceof SettingFragment) {
                ((SettingFragment) currentFragment).onBarcodePowerOnBeepChanged(isPowerOnBeep);
            }
        });
    }

    @Override
    public void onReaderInfoReceived(Map<String, String> info) {
        runOnUiThread(() -> {
            Fragment currentFragment = getCurrentFragment();
            if (currentFragment instanceof InfoFragment) {
                String model = info.get(DeviceConst.KEY_INFO_MODEL_NAME);
                String hwVersion = info.get(DeviceConst.KEY_INFO_HARDWARE_VERSION);
                String fwVersion = info.get(DeviceConst.KEY_INFO_FIRMWARE_VERSION);

                ((InfoFragment) currentFragment).setReaderInfo(model, hwVersion, fwVersion);
            }
        });
    }

    @Override
    public void onSettingParamReceived(Map<String, Boolean> parameters){
        Log.d(TAG, "onSettingParamReceived: " +
                "ScanBeep : " + parameters.get(DeviceConst.KEY_PARAMETER_SCAN_BEEP) +
                "ScanVibration : " + parameters.get(DeviceConst.KEY_PARAMETER_SCAN_VIBRATION) +
                "ScanAimer : " + parameters.get(DeviceConst.KEY_PARAMETER_SCAN_AIMER) +
                "LED : " + parameters.get(DeviceConst.KEY_PARAMETER_LED) +
                "PowerOnBeep : " + parameters.get(DeviceConst.KEY_PARAMETER_POWER_ON_BEEP));
    }

    @Override
    public void onGetBarcodePowerOnBeepReceived(boolean isPowerOnBeep) {

    }

    @Override
    public void onDeviceManagerError(String message) {
        Log.d("MainFragment", "onDeviceManagerError: " + message);

        if(message.equals("Power is already on.")) {
            onCompletedPreparation(true);
            return;
        }

        runOnUiThread(() -> {
            Fragment currentFragment = getCurrentFragment();
            if (currentFragment instanceof BaseFragment) {
                ((BaseFragment) currentFragment).onDeviceError(message);

            } else if (currentFragment instanceof SettingFragment) {
                ((SettingFragment) currentFragment).onDeviceError(message);
            }
        });
    }
}