package jp.co.asterisk.asreader.a23d.demoapp.fragments;

import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.navigation.fragment.NavHostFragment;

import java.util.ArrayList;
import java.util.Timer;
import java.util.TimerTask;

import jp.co.asterisk.asreader.a23d.demoapp.MainActivity;
import jp.co.asterisk.asreader.a23d.demoapp.R;
import jp.co.asterisk.asreader.a23d.demoapp.databinding.FragmentBarcodeScanBinding;
import jp.co.asterisk.asreader.a23d.demoapp.util.SharedPreferencesHelper;
import jp.co.asterisk.asreader.a23d.demoapp.util.TagData;
import jp.co.asterisk.asreader.a23d.demoapp.util.TagDataListAdapter;
import jp.co.asterisk.asreader.a23d.sdk.AsReader;

import jp.co.asterisk.asreader.a23d.sdk.barcodemanager.BarcodeManagerCallback;

public class BarcodeScanFragment extends BaseFragment implements BarcodeManagerCallback {

    private FragmentBarcodeScanBinding binding;

    private ArrayList<TagData> tagDataList;
    private TagDataListAdapter tagDataListAdapter;
    private ProgressDialogFragment progressDialogFragment;

    final Handler handler = new Handler(Looper.getMainLooper());
    private boolean isViewCreated = false;

    private Timer timer;
    private long elapsed = 0;
    private final long TIMER_INTERVAL = 1000;
    private final long TIMER_TIME_OUT = 1000 * 60 * 5 ;

    boolean isDialogShow = false;

    @Override
    public View onCreateView( @NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState ) {
        // Inflate the layout for this fragment
        binding = FragmentBarcodeScanBinding.inflate(inflater, container, false);
        return binding.getRoot();
    }

    public interface onClickTestButtonListener {
        void clickTestButton();
    }

    public void onViewCreated(@NonNull View view, Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        Log.d("MainFragment", "onViewCreated");

        setTitle(getString(R.string.app_name));
        setBackButtonVisibility(View.VISIBLE);
        setRightButtonEnabled(false);
        setRightButtonVisibility(View.INVISIBLE);

        // Connection Type set OTG
        SharedPreferencesHelper.getInstance().setReaderConnectType(false);
        binding.powerToggleButton.setOnClickListener(onClickListener);
        binding.powerToggleButton.setChecked(false);
        binding.powerToggleButton.setFocusable(false);

        binding.readButton.setOnClickListener(onClickListener);
        binding.readButton.setFocusable(false);

        binding.clearButton.setOnClickListener(onClickListener);
        binding.clearButton.setFocusable(false);

        binding.stopButton.setOnClickListener(onClickListener);
        binding.stopButton.setFocusable(false);

        tagDataList = ((MainActivity)requireActivity()).getTagDataList();

        tagDataListAdapter = new TagDataListAdapter(requireContext());
        tagDataListAdapter.setTagDataList(tagDataList);
        binding.tagListView.setAdapter(tagDataListAdapter);
        binding.tagListView.setFocusable(false);
        binding.tagListView.setFocusableInTouchMode(false);
        binding.tagListView.setOnItemClickListener((parent, view1, position, id) -> {
            AsReader.getInstance().getBarcodeManager().stopDecode();

            Bundle args = new Bundle();
            args.putString("selectTag", tagDataList.get(position).getName());
            NavHostFragment.findNavController(BarcodeScanFragment.this)
                    .navigate(R.id.action_BarcodeScanFragment_to_BarcodeAccessFragment, args);
        });

        binding.tagCountTextView.setText(tagDataListAdapter.getCount() + " tags");

        isViewCreated = true;
    }

    @Override
    public void onStart(){
        super.onStart();
        Log.d("BarcodeScanFragment", "onStart");
        enableActionBar(true);

        setUIEnable(AsReader.getInstance().getDeviceManager().isPowerOn());
        AsReader.getInstance().getBarcodeManager().setCallback(this);
    }

    @Override
    public void onResume() {
        super.onResume();
        connectProcess();
    }

    @Override
    public void onPause() {
        Log.d("MainFragment", "onPause");
        hideProgressDialog();
        AsReader.getInstance().getBarcodeManager().stopDecode();
        stopTimer();

        super.onPause();
    }

    private final View.OnClickListener onClickListener = new View.OnClickListener() {
        @Override
        public void onClick(View v) {
            if (v == binding.powerToggleButton) {
                //Processing not to send the same command while processing the 'power on' command.
                // Enable when message 'reset reader 0x08'
                Log.i("onClick", "PowerToggle : " + binding.powerToggleButton.isChecked() );
                setPowerOnOff(binding.powerToggleButton.isChecked());

            } else if (v == binding.readButton) {
                AsReader.getInstance().getBarcodeManager().startAutoDecode();
                startTimer();

            } else if (v == binding.stopButton) {
                AsReader.getInstance().getBarcodeManager().stopDecode();

            } else if (v == binding.clearButton) {
                tagDataList.clear();
                tagDataListAdapter.notifyDataSetChanged();
                binding.tagCountTextView.setText("0  tags");
            }
        }
    };

    private void setPowerOnOff(boolean power) {
        boolean beep = SharedPreferencesHelper.getInstance().getBeep();
        boolean vib = SharedPreferencesHelper.getInstance().getVibration();
        boolean led = SharedPreferencesHelper.getInstance().getLed();
        boolean aimer = SharedPreferencesHelper.getInstance().getAimer();
        boolean powerOnBeep = SharedPreferencesHelper.getInstance().getPowerOnBeep();

        AsReader.getInstance().getDeviceManager().setPowerWithOption(power, beep, vib, led, aimer, powerOnBeep);
    }

    private synchronized void updateTagDataList(String data) {
        int count = tagDataListAdapter.getCount();
        boolean itemExists = false;
        for (int i=0; i<count; i++) {
            if (data.equals(tagDataList.get(i).getName())) {
                tagDataList.get(i).setCount(tagDataList.get(i).getCount() + 1);
                itemExists = true;
                break;
            }
        }

        if (!itemExists) {
            tagDataList.add(new TagData(data));
        }
        tagDataListAdapter.notifyDataSetChanged();
        binding.tagCountTextView.setText(tagDataListAdapter.getCount() + " tags");
    }
    private void showProgressDialog() {
        if (progressDialogFragment == null && !isDialogShow) {
            progressDialogFragment = ProgressDialogFragment.newInstance("Please wait..");
            progressDialogFragment.show(getParentFragmentManager(), "dialog");
            isDialogShow = true;
        }
    }

    private void hideProgressDialog() {
        if (progressDialogFragment != null) {
            progressDialogFragment.dismiss();
            progressDialogFragment = null;
            isDialogShow = false;

            FragmentManager mgr = getActivity().getSupportFragmentManager();
            Fragment fragment = mgr.findFragmentByTag("dialog");
            if( fragment instanceof ProgressDialogFragment ) {
                ((ProgressDialogFragment)fragment).dismiss();
            }
        }
    }

    private void setUIEnable(boolean enable){
        if (enable) {
            binding.deviceStatusTextView.setText(R.string.Connected);
        } else {
            binding.deviceStatusTextView.setText(R.string.Disconnected);
        }
        setBottomUIEnable(enable);
    }

    private void setBottomUIEnable(boolean enable){
        binding.powerToggleButton.setChecked(enable);
        binding.readButton.setEnabled(enable);
        binding.clearButton.setEnabled(enable);
        binding.stopButton.setEnabled(enable);
    }

    private void setHeaderUIEnable(boolean enable){
        if( enable ) {
            binding.tagCountTextView.setVisibility(View.VISIBLE);
            binding.deviceStatusTextView.setVisibility(View.VISIBLE);
            binding.batteryTextView.setVisibility(View.VISIBLE);
        } else {
            binding.tagCountTextView.setVisibility(View.INVISIBLE);
            binding.deviceStatusTextView.setVisibility(View.INVISIBLE);
            binding.batteryTextView.setVisibility(View.INVISIBLE);
        }
    }

    private void connectProcess() {
        handler.postDelayed(() -> {
            // UIスレッド
            boolean isConnected = AsReader.getInstance().getDeviceManager().isConnected();
            if ( isConnected && isViewCreated ) {
                if( !AsReader.getInstance().getDeviceManager().isPowerOn() &&
                    !SharedPreferencesHelper.getInstance().getHidMode() ) {
                    binding.deviceStatusTextView.setText(R.string.Connected);
                    showProgressDialog();
                    setPowerOnOff(true);
                }
            }
        }, 10);
    }

    // Keep screen on if true
    private void keepScreenOnMode(Activity activity, boolean keepScreenOn){
        if (getActivity()!=null){
            getActivity().runOnUiThread(() -> {
                if(keepScreenOn){
                    activity.getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
                } else {
                    activity.getWindow().clearFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
                }
            });
        }
    }

    private void startTimer(){
        keepScreenOnMode(getActivity(), true);

        if(timer != null)
            return;

        TimerTask task = new TimerTask(){
            @Override
            public void run() {
                elapsed += TIMER_INTERVAL;
                if(elapsed >= TIMER_TIME_OUT){
                    stopTimer();
                }
            }
        };
        timer = new Timer();
        timer.scheduleAtFixedRate(task, TIMER_INTERVAL, TIMER_INTERVAL);
    }

    private void stopTimer(){
        keepScreenOnMode(getActivity(), false);

        if(timer != null){
            timer.cancel();
            timer.purge();
            timer = null;
            elapsed = 0;
        }
    }

    @Override
    public void onRightButtonClick() {
//        isMorePage = true;
        NavHostFragment.findNavController(BarcodeScanFragment.this).navigate(R.id.action_BarcodeScanFragment_to_MenuFragment);
    }

    @Override
    public void onBackPressed() {
        super.onBackPressed();
        //setPowerOnOff(false);
    }

    @Override
    public void onConnect(boolean isConnect) {
        Log.d("MainFragment", "onConnect: " + isConnect);

        if (!isConnect) {
            if( isViewCreated ) {
                binding.deviceStatusTextView.setText(R.string.Disconnected);
                setBottomUIEnable(false);
                hideProgressDialog();
            }
        }
        setPowerOnOff(isConnect);
    }

    @Override
    public void onCompletedPreparation(boolean isPowerOn) {
        hideProgressDialog();
        setUIEnable(isPowerOn);
    }

    @Override
    public void onDeviceError(String message) {
        hideProgressDialog();
    }

    public void onBatteryStateReceived(int battery) {
        requireActivity().runOnUiThread(() -> binding.batteryTextView.setText(battery + "%"));
    }

    // *******************************************************
    //
    //  BarcodeManagerCallback
    //
    // *******************************************************
    @Override
    public void onReceivedScanData(String data) {
        Log.d("MainFragment", "onReceivedData: " + data);
        requireActivity().runOnUiThread(() -> updateTagDataList(data));
        
        // Reset 5 minutes timer
        elapsed = 0;
    }

    @Override
    public void onBarcodeManagerError(String message) {
        Log.d("MainFragment", "onBarcodeManagerError: " + message);
    }
}