package jp.co.asterisk.asreader.a23d.demoapp.fragments;

import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.navigation.fragment.NavHostFragment;

import jp.co.asterisk.asreader.a23d.demoapp.R;
import jp.co.asterisk.asreader.a23d.demoapp.databinding.FragmentMenuBinding;
import jp.co.asterisk.asreader.a23d.demoapp.util.SharedPreferencesHelper;
import jp.co.asterisk.asreader.a23d.sdk.AsReader;
import jp.co.asterisk.asreader.a23d.sdk.barcodemanager.BarcodeConst;
import jp.co.asterisk.asreader.a23d.sdk.barcodemanager.BarcodeManagerCallback;

public class MenuFragment extends BaseFragment implements BarcodeManagerCallback {
    private final String TAG = "MenuFragment";

    private FragmentMenuBinding binding;

    private ProgressDialogFragment progressDialogFragment = null;
    private boolean isCreateView = false;
    final Handler handler = new Handler(Looper.getMainLooper());

    boolean isDialogShow = false;

    private final View.OnClickListener onClickListener = new View.OnClickListener() {
        @Override
        public void onClick(View v) {
            if (v == binding.moveScanButton) {
                Log.i(TAG, "onClick Move Scan Screen: " );
                NavHostFragment.findNavController(MenuFragment.this).navigate(R.id.action_MenuFragment_to_BarcodeScanFragment);

            } else if (v == binding.moveInfoButton) {
                Log.i(TAG, "onClick Move Information Screen: " );
                NavHostFragment.findNavController(MenuFragment.this).navigate(R.id.action_MenuFragment_to_InfoFragment);

            } else if (v == binding.moveSettingButton) {
                Log.i(TAG, "onClick Move Setting Screen: " );
                NavHostFragment.findNavController(MenuFragment.this).navigate(R.id.action_MenuFragment_to_SettingFragment);
            }
        }
    };

    @Override
    public View onCreateView( @NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState ) {
        // Inflate the layout for this fragment
        binding = FragmentMenuBinding.inflate(inflater, container, false);
        return binding.getRoot();
    }

    public void onViewCreated(@NonNull View view, Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        Log.d(TAG, "onViewCreated");

        setTitle("");
        setRightButtonText("");

        // Connection Type set OTG
        SharedPreferencesHelper.getInstance().setReaderConnectType(false);

        binding.moveScanButton.setOnClickListener(onClickListener);
        binding.moveScanButton.setFocusable(false);
        binding.moveScanButton.setEnabled(false);

        binding.moveInfoButton.setOnClickListener(onClickListener);
        binding.moveInfoButton.setFocusable(false);
        binding.moveInfoButton.setEnabled(false);

        binding.moveSettingButton.setOnClickListener(onClickListener);
        binding.moveSettingButton.setFocusable(false);
        binding.moveSettingButton.setEnabled(false);

        setRightButtonEnabled(false);

        isCreateView = true;

        if( AsReader.getInstance().getDeviceManager().isConnected() ) {
            binding.moveScanButton.setEnabled(true);
            binding.moveSettingButton.setEnabled(true);
            binding.moveInfoButton.setEnabled(true);
        }
    }

    @Override
    public void onStart(){
        super.onStart();
        Log.d(TAG, "onStart");
        setBackButtonVisibility(View.INVISIBLE);
        setRightButtonVisibility(View.VISIBLE);
        enableActionBar(false);
        AsReader.getInstance().getBarcodeManager().setCallback(this);

        if( AsReader.getInstance().getDeviceManager().isConnected() ){
            AsReader.getInstance().getBarcodeManager().getHIDMode();

            if (AsReader.getInstance().getDeviceManager().isPowerOn()) {
                handler.postDelayed(() -> AsReader.getInstance().getDeviceManager().setPower(false),300);
            }
        }
    }

    @Override
    public void onResume() {
        super.onResume();
    }

    @Override
    public void onPause() {
        Log.d(TAG, "onPause");
        hideProgressDialog();
        super.onPause();
    }

    private void checkHIDModeProgress(){
//        handler.postDelayed(() -> {
//            // UIスレッド
//
//        }, 80);

        boolean isConnected = AsReader.getInstance().getDeviceManager().isConnected();
        if ( isConnected && isCreateView ) {
//            showProgressDialog();
            AsReader.getInstance().getBarcodeManager().getHIDMode();
        }
    }

    private void showProgressDialog() {
        if (progressDialogFragment == null && !isDialogShow) {
            progressDialogFragment = ProgressDialogFragment.newInstance("Please wait..");
            progressDialogFragment.show(getParentFragmentManager(), "dialog");
            isDialogShow = true;
        }
    }

    private void hideProgressDialog() {
        if (progressDialogFragment != null && isAdded()) {
            progressDialogFragment.dismiss();
            progressDialogFragment = null;
            isDialogShow = false;
        }
    }

    // Keep screen on if true
    private void keepScreenOnMode(Activity activity, boolean keepScreenOn){
        if (getActivity()!=null){
            getActivity().runOnUiThread(() -> {
                if(keepScreenOn){
                    activity.getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
                } else {
                    activity.getWindow().clearFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
                }
            });
        }
    }

    // Activity Override method
    @Override
    public void onRightButtonClick() {
    }

    @Override
    public void onBackPressed() {
        super.onBackPressed();
    }

    // DeviceManager Callback
    @Override
    public void onConnect(boolean isConnect) {
        Log.d("MainFragment", "onConnect: " + isConnect);

        if (isConnect) {
            checkHIDModeProgress();
//            showToast("Connected");
        } else {
            binding.moveScanButton.setEnabled(false);
            binding.moveSettingButton.setEnabled(false);
            binding.moveInfoButton.setEnabled(false);
//            hideProgressDialog();
//            showToast("Disconnected");
        }
    }

    @Override
    public void onCompletedPreparation(boolean isPowerOn) {
//        hideProgressDialog();
    }

    @Override
    public void onSetPowerOnSuccess() {
//        hideProgressDialog();
    }

    @Override
    public void onDeviceError(String message) {
//        hideProgressDialog();
    }

    // BarcodeManager Callback
    @Override
    public void onReceivedScanData(String data) {

    }

    @Override
    public void onHIDSettingResponse(boolean status, BarcodeConst.ResponseCode code) {


        getActivity().runOnUiThread(() -> {
            switch (code){
                case GET_HID_MODE:
                    if(status){
                        binding.moveScanButton.setEnabled(false);
                    } else {
                        binding.moveScanButton.setEnabled(true);
                    }
                    binding.moveSettingButton.setEnabled(true);
                    binding.moveInfoButton.setEnabled(true);
                    break;
            }
//            hideProgressDialog();
        });
//        Handler handler = new Handler(Looper.getMainLooper());
//        handler.postDelayed(() -> {
//            // UIスレッド
//            switch (code){
//                case GET_HID_MODE:
//                    if(status){
//                        binding.moveScanButton.setEnabled(false);
//                    } else {
//                        binding.moveScanButton.setEnabled(true);
//                    }
//                    binding.moveSettingButton.setEnabled(true);
//                    binding.moveInfoButton.setEnabled(true);
//                    break;
//            }
//            hideProgressDialog();
//        }, 50);
    }

    @Override
    public void onFactoryDefaultSuccess() {

    }

    @Override
    public void onBarcodeManagerError(String message) {

    }
}
