package jp.co.asterisk.asreader.a23d.demoapp.fragments;

import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.navigation.fragment.NavHostFragment;

import jp.co.asterisk.asreader.a23d.demoapp.R;
import jp.co.asterisk.asreader.a23d.demoapp.databinding.FragmentSettingBinding;
import jp.co.asterisk.asreader.a23d.demoapp.util.SharedPreferencesHelper;
import jp.co.asterisk.asreader.a23d.sdk.AsReader;
import jp.co.asterisk.asreader.a23d.sdk.barcodemanager.BarcodeConst;
import jp.co.asterisk.asreader.a23d.sdk.barcodemanager.BarcodeManagerCallback;

public class SettingFragment extends BaseFragment implements BarcodeManagerCallback {

    private ProgressDialogFragment progressDialogFragment;
    private FragmentSettingBinding binding;

    private boolean isCreateView = false;
    final Handler handler = new Handler(Looper.getMainLooper());

    @Override
    public View onCreateView( @NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState ) {
        // Inflate the layout for this fragment
        binding = FragmentSettingBinding.inflate(inflater, container, false);
        return binding.getRoot();
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        setTitle(getString(R.string.setting));
        setBackButtonVisibility(View.VISIBLE);
        setRightButtonVisibility(View.INVISIBLE);

        binding.beepToggleButton.setChecked(SharedPreferencesHelper.getInstance().getBeep());
        binding.beepToggleButton.setOnClickListener(clickListener);

        binding.vibrationToggleButton.setChecked(SharedPreferencesHelper.getInstance().getVibration());
        binding.vibrationToggleButton.setOnClickListener(clickListener);

        binding.ledToggleButton.setChecked(SharedPreferencesHelper.getInstance().getLed());
        binding.ledToggleButton.setOnClickListener(clickListener);

        binding.aimerToggleButton.setChecked(SharedPreferencesHelper.getInstance().getAimer());
        binding.aimerToggleButton.setOnClickListener(clickListener);

        binding.powerBeepToggleButton.setChecked(SharedPreferencesHelper.getInstance().getPowerOnBeep());
        binding.powerBeepToggleButton.setOnClickListener(clickListener);

        binding.hidModeToggleButton.setChecked(SharedPreferencesHelper.getInstance().getHidMode());
        binding.hidModeToggleButton.setOnClickListener(clickListener);

        binding.defaultSettingButton.setOnClickListener(clickListener);

        isCreateView = true;

        checkHIDModeProgress();
    }

    @Override
    public void onStart() {
        super.onStart();
        Log.d("SettingFragment", "onStart");
        enableActionBar(true);
        AsReader.getInstance().getBarcodeManager().setCallback(this);
    }

    @Override public void onResume(){
        super.onResume();
    }
    @Override
    public void onPause(){
        super.onPause();
        hideProgressDialog();
    }

    private void setButtonEnable(boolean isEnabled){
        binding.beepToggleButton.setEnabled(isEnabled);
        binding.vibrationToggleButton.setEnabled(isEnabled);
        binding.ledToggleButton.setEnabled(isEnabled);
        binding.aimerToggleButton.setEnabled(isEnabled);
        binding.powerBeepToggleButton.setEnabled(isEnabled);
        binding.hidModeToggleButton.setEnabled(isEnabled);

        if(isEnabled) {
            binding.beepToggleButton.setChecked(SharedPreferencesHelper.getInstance().getBeep());
            binding.vibrationToggleButton.setChecked(SharedPreferencesHelper.getInstance().getVibration());
            binding.ledToggleButton.setChecked(SharedPreferencesHelper.getInstance().getLed());
            binding.aimerToggleButton.setChecked(SharedPreferencesHelper.getInstance().getAimer());
            binding.powerBeepToggleButton.setChecked(SharedPreferencesHelper.getInstance().getPowerOnBeep());
            binding.hidModeToggleButton.setChecked(SharedPreferencesHelper.getInstance().getHidMode());
            binding.defaultSettingButton.setEnabled(!SharedPreferencesHelper.getInstance().getHidMode());
        }
    }

    private final View.OnClickListener clickListener = new View.OnClickListener() {
        @Override
        public void onClick(View v) {

            if (v == binding.beepToggleButton ||
                    v == binding.vibrationToggleButton ||
                    v == binding.ledToggleButton ||
                    v == binding.aimerToggleButton) {
                setButtonEnable(false);
                if( !SharedPreferencesHelper.getInstance().getHidMode() ) {
                    showProgressDialog();
                }
                changeSetting();
            } else if (v == binding.powerBeepToggleButton) {
                if( !SharedPreferencesHelper.getInstance().getHidMode() ) {
                    setButtonEnable(false);
                    showProgressDialog();
                }
                boolean powerOnBeep = binding.powerBeepToggleButton.isChecked();
                AsReader.getInstance().getDeviceManager().setPowerOnBeep(powerOnBeep);
                SharedPreferencesHelper.getInstance().setPowerOnBeep(powerOnBeep);

            } else if(v==binding.hidModeToggleButton) {
                setButtonEnable(false);
                showProgressDialog();

                if (binding.hidModeToggleButton.isChecked()) {
                    // OTG Power Off
                    setPowerOnOff(false);
                    // HID Mode On - Delay 0.1
                    handler.postDelayed(() -> AsReader.getInstance().getBarcodeManager().setHIDMode(true), 300);
                } else {
                    // HID Mode Off
                    AsReader.getInstance().getBarcodeManager().setHIDMode(false);
                    handler.postDelayed(() -> setPowerOnOff(false), 300);

                    // OTG Power On - Delay 0.5
                    handler.postDelayed(() -> setPowerOnOff(true), 600);
                }

                SharedPreferencesHelper.getInstance().setHidMode(binding.hidModeToggleButton.isChecked());
            }
            else if (v == binding.defaultSettingButton) {
                if (!AsReader.getInstance().getDeviceManager().isPowerOn()) {
                    showToast("電源が入っていません");
                    return;
                }

                binding.defaultSettingButton.setEnabled(false);

                showProgressDialog();

                AsReader.getInstance().getBarcodeManager().factoryDefault();
                // Default : true
                SharedPreferencesHelper.getInstance().setHidMode(false); // sdk mode
                SharedPreferencesHelper.getInstance().setBeep(true);
                SharedPreferencesHelper.getInstance().setVibration(true);
                SharedPreferencesHelper.getInstance().setLed(true);
                SharedPreferencesHelper.getInstance().setAimer(true);
                SharedPreferencesHelper.getInstance().setPowerOnBeep(true);
            }
        }
    };

    private void setPowerOnOff(boolean power) {
        boolean beep = SharedPreferencesHelper.getInstance().getBeep();
        boolean vib = SharedPreferencesHelper.getInstance().getVibration();
        boolean led = SharedPreferencesHelper.getInstance().getLed();
        boolean aimer = SharedPreferencesHelper.getInstance().getAimer();
        boolean powerOnBeep = SharedPreferencesHelper.getInstance().getPowerOnBeep();

        AsReader.getInstance().getDeviceManager().setPowerWithOption(power, beep, vib, led, aimer, powerOnBeep);
    }

    private void changeSetting() {
        boolean beep = binding.beepToggleButton.isChecked();
        boolean vib = binding.vibrationToggleButton.isChecked();
        boolean aimer = binding.aimerToggleButton.isChecked();
        boolean led = binding.ledToggleButton.isChecked();

        AsReader.getInstance().getDeviceManager().setReaderSettings(beep, vib, aimer, led);

        SharedPreferencesHelper.getInstance().setBeep(beep);
        SharedPreferencesHelper.getInstance().setVibration(vib);
        SharedPreferencesHelper.getInstance().setAimer(aimer);
        SharedPreferencesHelper.getInstance().setLed(led);
    }

    private void showProgressDialog() {
        progressDialogFragment = ProgressDialogFragment.newInstance("Please wait..");
        progressDialogFragment.show(getParentFragmentManager(), "dialog");
    }

    private void hideProgressDialog() {
        if (progressDialogFragment != null) {
            progressDialogFragment.dismiss();
        }
        setButtonEnable(true);
    }

    private void checkHIDModeProgress(){
        handler.postDelayed(() -> {
            // UIスレッド
            boolean isConnected = AsReader.getInstance().getDeviceManager().isConnected();
            if (isConnected && isCreateView) {
                showProgressDialog();
                AsReader.getInstance().getBarcodeManager().getHIDMode();
            }
        }, 50);
    }

    @Override
    public void onSetReaderSettingSuccess() {
        hideProgressDialog();
        showToast("Success");
    }

    @Override
    public void onBarcodePowerOnBeepChanged(boolean isPowerOnBeep) {
        hideProgressDialog();
        showToast("Success");
    }

    @Override
    public void onConnect(boolean isConnect) {
        Log.d("SettingFragment", "onConnect : " + isConnect);

        if (!isConnect) {
            NavHostFragment.findNavController(SettingFragment.this).navigate(R.id.action_SettingFragment_to_MenuFragment);
        }
    }

    @Override
    public void onHIDSettingResponse(final boolean status, BarcodeConst.ResponseCode code) {
        requireActivity().runOnUiThread(() -> {
            switch (code) {
                case GET_HID_MODE:
                    hideProgressDialog();
                    Log.d("MainFragment", "onHidModeChanged: " + status);
                    showToast("onHidModeChanged Mode : " + status);
                    SharedPreferencesHelper.getInstance().setHidMode(status);
                    binding.defaultSettingButton.setEnabled(!status);

                    if (!status) {
                        if (!AsReader.getInstance().getDeviceManager().isPowerOn()) {
                            setPowerOnOff(true);
                        }
                    }
                    break;

                case SET_HID_MODE:
                    Log.d("MainFragment", "onGetHidModeReceived: " + status);
                    showToast("onHidModeChanged Mode : " + status);
                    SharedPreferencesHelper.getInstance().setHidMode(status);
                    if(status) {
                        //AsReader.getInstance().getDeviceManager().modeChangeInitialise();
                        hideProgressDialog();
                    }
                    break;

                case GET_HID_SLEEP_MODE:
                case SET_HID_SLEEP_MODE:
                    break;
            }
        });
    }

    @Override
    public void onFactoryDefaultSuccess() {
        requireActivity().runOnUiThread(() -> {
            AsReader.getInstance().getDeviceManager().setReaderSettings(
                    SharedPreferencesHelper.getInstance().getBeep(),
                    SharedPreferencesHelper.getInstance().getVibration(),
                    SharedPreferencesHelper.getInstance().getAimer(),
                    SharedPreferencesHelper.getInstance().getLed());

            binding.defaultSettingButton.setEnabled(true);
            hideProgressDialog();
            showToast("Success");
        });
    }

    @Override
    public void onCompletedPreparation(boolean isPowerOn) {
        binding.defaultSettingButton.setEnabled(isPowerOn);
        if(isPowerOn) {
            hideProgressDialog();
        }
    }

    @Override
    public void onBarcodeManagerError(String message) {
        hideProgressDialog();
    }

    @Override
    public void onDeviceError(String message) {
        hideProgressDialog();
    }
}